#ifndef LIBKYDEVICESEC_H
#define LIBKYDEVICESEC_H

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <ncurses.h>
#include <panel.h>
#include <locale.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/inotify.h>
#include <selinux/selinux.h>
#include<getopt.h>
#include <dlfcn.h>

#include <string>
#include <vector>
#include <iostream>

#include "usbguard/ConfigFile.hpp"
#include "usbguard/IPCClient.hpp"
#include "usbguard/Rule.hpp"
#include "usbguard/RuleParser.hpp"

#if 0
namespace usbguard
{
#endif
  /**
   * @brief 添加usb设备管控策略
   *
   * @param rule_spec 待添加设备管控策略条目
   * @param parent_id 前一条设备管控策略条目id， 默认值usbguard::Rule::LastID
   * @return > 0， success, 返回新添加管控策略条目序号id
   *         other, failed
   */
  int kdk_usbguard_appendRule(const std::string& rule_spec);

  /**
   * @brief 删除usb设备管控策略
   *
   * @param id 待删除设备管控策略条目序号
   * @return 0, success, 成功删除一条策略;-1:删除失败
   */
  int kdk_usbguard_removeRule(uint32_t id);

  /**
   * @brief 添加指定usb设备管控策略
   *
   * @param vid 指定设备vid
   * @param pid 指定设备pid
   * @param name 指定设备名, 可为空
   * @param serial 指定设备序列号,可为空
   * @param interface 指定设备接口
   * @param target 策略类别: Allow, Block, Reject
   * @return > 0， success, 返回新添加管控策略条目序号id
   *         other, failed
   */
  int kdk_usbguard_appendRule_specific(std::string& vid, std::string& pid, std::string name, std::string serial, std::string interface, int target);

  /**
   * @brief 删除指定usb设备管控策略
   *
   * @param pid 指定设备pid
   * @param vid 指定设备vid
   * @param name 指定设备名, 可为空
   * @param serial 指定设备序列号,可为空
   * @return = 0， success
   * @return < 0， 指定设备策略不存在,或操作失败
   */
  int kdk_usbguard_removeRule_specific(std::string& vid, std::string& pid, std::string name, std::string serial);

  /**
   * @brief 获取usb设备管控策略
   * @param rules 当前策略列表
   * @return = 0, success
   * @return < 0，操作失败
   */
  int kdk_usbguard_getRules(std::vector<usbguard::Rule>& rules);

  /**
   * @brief 获取指定usb设备管控策略
   * @param pid 指定设备pid
   * @param vid 指定设备vid
   * @param name 指定设备名, 可为空
   * @param serial 指定设备序列号,可为空
   * @param interface 指定设备接口
   * @return返回策略类别: Allow=0, Block=1, Reject=2
   * @return < 0，failed
   */
  int  kdk_usbguard_getRule_specific(std::string& vid, std::string& pid, std::string name, std::string serial, std::string interface);

  /**
   * @brief 获取当前已连接外设usb设备管控状态
   * @param devices 当前设备列表
   * @return = 0, success
   * @return < 0，操作失败
   */
  int kdk_usbguard_getDevices(std::vector<usbguard::Rule>& devices);

  /**
   * @brief 改变指定usb设备的管控状态
   * @param id 指定设备在设备列表中顺序号
   * @param target 将要改为的策略类别: Allow=0, Block=1, Reject=2
   * @param permanent是否永久改变该usb设备的策略,当为true时会生成一条策略条目写入策略文件
   * @return = 0， success
   * @return < 0， 指定设备策略不存在,或操作失败
   */
  int kdk_usbguard_applyDevicePolicy(uint32_t id, usbguard::Rule::Target target, bool permanent);

  /**
   * @brief 修改一条策略条目
   *
   * @param id 待修改更新设备管控策略条目序号
   * @param target 策略类别: Allow=0, Block=1, Reject=2
   * @return 0, success, 成功更新一条策略;-1:失败
   * 调用后策略序号可能会改变,需重新获取策略列表，以获取更新后的序号;
   */
  int kdk_usbguard_upsertRule(int id, int target);

  /**
   * @brief 导入usb设备管控策略
   *
   * @param path  待导入系统的设备管控策略文件
   * @return 0, success, 成功更新系统策略;-1:失败
   * 调用后策略，设备序号可能会改变,需重新获取策略列表，设备列表;
   */
   int kdk_usbguard_load_Rules_path(std::string& path);

  /**
   * @brief 导入usb设备管控策略
   *
   * @param stream  待导入系统的设备管控策略文件流
   * @return 0, success, 成功更新系统策略;-1:失败
   * 调用后策略，设备序号可能会改变,需重新获取策略列表，设备列表;
   */   
   int kdk_usbguard_load_Rules_stream(std::iostream& stream);

#if 0
} /* namespace usbguard */
#endif

#endif // LIBKYDEVICESEC_H
